"""
mbed SDK
Copyright (c) 2011-2015 ARM Limited

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import os
import mock
import six
import sys
import unittest

from mbed_greentea import cmake_handlers, tests_spec

class GreenteaCmakeHandlers(unittest.TestCase):
    """ Basic true asserts to see that testing is executed
    """

    def setUp(self):
        self.ctesttestfile = """# CMake generated Testfile for
# Source directory: c:/Work2/mbed-client/test
# Build directory: c:/Work2/mbed-client/build/frdm-k64f-gcc/test
#
# This file includes the relevant testing commands required for
# testing this directory and lists subdirectories to be tested as well.
add_test(mbed-client-test-mbedclient-smokeTest "mbed-client-test-mbedclient-smokeTest")
add_test(mbed-client-test-helloworld-mbedclient "mbed-client-test-helloworld-mbedclient")
"""

    def tearDown(self):
        pass

    def test_example(self):
        self.assertEqual(True, True)
        self.assertNotEqual(True, False)

    def test_parse_ctesttestfile_line(self):
        LINK_TARGET = '/dir/to/target'
        BINARY_TYPE = '.bin'

        result = {}
        no_skipped_lines = 0
        for line in self.ctesttestfile.splitlines():
            line_parse = cmake_handlers.parse_ctesttestfile_line(LINK_TARGET, BINARY_TYPE, line, verbose=True)
            if line_parse:
                test_case, test_case_path = line_parse
                result[test_case] = test_case_path
            else:
                no_skipped_lines += 1

        self.assertIn('mbed-client-test-mbedclient-smokeTest', result)
        self.assertIn('mbed-client-test-helloworld-mbedclient', result)

        for test_case, test_case_path in result.items():
            self.assertEqual(test_case_path.startswith(LINK_TARGET), True)
            self.assertEqual(test_case_path.endswith(BINARY_TYPE), True)

        self.assertEqual(len(result), 2)        # We parse two entries
        self.assertEqual(no_skipped_lines, 6)   # We skip six lines in this file

    def test_load_ctest_testsuite_missing_link_target(self):
        null_link_target = None
        null_suite = cmake_handlers.load_ctest_testsuite(null_link_target)
        self.assertEqual(null_suite, {})


    def test_load_ctest_testsuite(self):
        root_path = os.path.dirname(os.path.realpath(__file__))
        emty_path = os.path.join(root_path, "resources", "empty")
        full_path = os.path.join(root_path, "resources", "not-empty")

        # Empty LINK_TARGET
        empty_link_target = emty_path
        empty_suite = cmake_handlers.load_ctest_testsuite(empty_link_target)
        self.assertEqual(empty_suite, {})

        # Not empty LINK_TARGET
        link_target = full_path
        test_suite = cmake_handlers.load_ctest_testsuite(link_target)
        self.assertIsNotNone(test_suite)
        self.assertIn('mbed-client-test-mbedclient-smokeTest', test_suite)
        self.assertIn('mbed-client-test-helloworld-mbedclient', test_suite)


    def test_list_binaries_for_targets(self):
        root_path = os.path.dirname(os.path.realpath(__file__))
        null_path = os.path.join(root_path, "resources", "empty", "test")
        full_path = os.path.join(root_path, "resources")

        def run_and_capture(path, verbose=False):
            old_stdout = sys.stdout
            sys.stdout = stdout_capture = six.StringIO()
            cmake_handlers.list_binaries_for_targets(build_dir=path, verbose_footer=verbose)
            sys.stdout = old_stdout

            return stdout_capture.getvalue()

        # Test empty target directory
        output = run_and_capture(null_path)
        self.assertTrue("no tests found in current location" in output)

        # Test empty target directory (Verbose output)
        output = run_and_capture(null_path, verbose=True)
        self.assertTrue("no tests found in current location" in output)
        self.assertTrue("Example: execute 'mbedgt -t TARGET_NAME -n TEST_NAME' to run test TEST_NAME for target TARGET_NAME" in output)

        # Test non-empty target directory
        output = run_and_capture(full_path)
        self.assertTrue("available tests for target" in output)


    def test_list_binaries_for_builds(self):
        root_path = os.path.dirname(os.path.realpath(__file__))
        spec_path = os.path.join(root_path, "resources", "test_spec.json")

        spec = tests_spec.TestSpec(spec_path)

        for verbose in [True, False]:
            # Capture logging output
            old_stdout = sys.stdout
            sys.stdout = stdout_capture = six.StringIO()
            cmake_handlers.list_binaries_for_builds(spec, verbose_footer=verbose)
            sys.stdout = old_stdout

            output = stdout_capture.getvalue()
            self.assertTrue("available tests for build 'K64F-ARM'" in output)
            self.assertTrue("available tests for build 'K64F-GCC'" in output)
            self.assertTrue("tests-example-1" in output)
            self.assertTrue("tests-example-2" in output)
            self.assertTrue("tests-example-7" in output)

            if verbose == True:
                self.assertTrue("Example: execute" in output)

if __name__ == '__main__':
    unittest.main()



"""
mbed SDK
Copyright (c) 2011-2015 ARM Limited

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import six
import sys
import unittest

from mbed_greentea import mbed_greentea_cli
from mbed_greentea.tests_spec import TestSpec

test_spec_def = {
    "builds": {
        "K64F-ARM": {
            "platform": "K64F",
            "toolchain": "ARM",
            "base_path": "./.build/K64F/ARM",
            "baud_rate": 115200,
            "tests": {
                "mbed-drivers-test-generic_tests":{
                    "binaries":[
                        {
                            "binary_type": "bootable",
                            "path": "./.build/K64F/ARM/mbed-drivers-test-generic_tests.bin"
                        }
                    ]
                },
                "mbed-drivers-test-c_strings":{
                    "binaries":[
                        {
                            "binary_type": "bootable",
                            "path": "./.build/K64F/ARM/mbed-drivers-test-c_strings.bin"
                        }
                    ]
                }
            }
        }
    }
}

class GreenteaCliFunctionality(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_get_greentea_version(self):
        version = mbed_greentea_cli.get_greentea_version()

        self.assertIs(type(version), str)

        a, b, c = version.split('.')

        self.assertEqual(a.isdigit(), True)
        self.assertEqual(b.isdigit(), True)
        self.assertEqual(c.isdigit(), True)

    def test_print_version(self):
        version = mbed_greentea_cli.get_greentea_version()

        old_stdout = sys.stdout
        sys.stdout = stdout_capture = six.StringIO()
        mbed_greentea_cli.print_version()
        sys.stdout = old_stdout

        printed_version = stdout_capture.getvalue().splitlines()[0]
        self.assertEqual(printed_version, version)

    def test_get_hello_string(self):
        version = mbed_greentea_cli.get_greentea_version()
        hello_string = mbed_greentea_cli.get_hello_string()

        self.assertIs(type(version), str)
        self.assertIs(type(hello_string), str)
        self.assertIn(version, hello_string)

    def test_get_local_host_tests_dir_invalid_path(self):
        test_path = mbed_greentea_cli.get_local_host_tests_dir("invalid-path")
        self.assertEqual(test_path, None)

    def test_get_local_host_tests_dir_valid_path(self):
        path = "."
        test_path = mbed_greentea_cli.get_local_host_tests_dir(path)
        self.assertEqual(test_path, path)

    def test_get_local_host_tests_dir_default_path(self):
        import os
        import shutil
        import tempfile

        curr_dir = os.getcwd()
        test1_dir = tempfile.mkdtemp()
        test2_dir = os.mkdir(os.path.join(test1_dir, "test"))
        test3_dir = os.mkdir(os.path.join(test1_dir, "test", "host_tests"))

        os.chdir(test1_dir)

        test_path = mbed_greentea_cli.get_local_host_tests_dir("")
        self.assertEqual(test_path, "./test/host_tests")

        os.chdir(curr_dir)
        shutil.rmtree(test1_dir)

    def test_create_filtered_test_list(self):
        test_spec = TestSpec()
        test_spec.parse(test_spec_def)
        test_build = test_spec.get_test_builds()[0]

        test_list = mbed_greentea_cli.create_filtered_test_list(test_build.get_tests(),
                                                                'mbed-drivers-test-generic_*',
                                                                None,
                                                                test_spec=test_spec)
        self.assertEqual(set(test_list.keys()), set(['mbed-drivers-test-generic_tests']))

        test_list = mbed_greentea_cli.create_filtered_test_list(test_build.get_tests(),
                                                                '*_strings',
                                                                None,
                                                                test_spec=test_spec)
        self.assertEqual(set(test_list.keys()), set(['mbed-drivers-test-c_strings']))

        test_list = mbed_greentea_cli.create_filtered_test_list(test_build.get_tests(),
                                                                'mbed*s',
                                                                None,
                                                                test_spec=test_spec)
        expected = set(['mbed-drivers-test-c_strings', 'mbed-drivers-test-generic_tests'])
        self.assertEqual(set(test_list.keys()), expected)

        test_list = mbed_greentea_cli.create_filtered_test_list(test_build.get_tests(),
                                                                '*-drivers-*',
                                                                None,
                                                                test_spec=test_spec)
        expected = set(['mbed-drivers-test-c_strings', 'mbed-drivers-test-generic_tests'])
        self.assertEqual(set(test_list.keys()), expected)

if __name__ == '__main__':
    unittest.main()



"""
mbed SDK
Copyright (c) 2011-2015 ARM Limited

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import os
import shutil
import tempfile
import unittest
from mbed_greentea import mbed_yotta_api

def generate_paths_and_write(data):
    # Generate some dummy temp directories
    curr_dir = os.getcwd()
    temp0_dir = tempfile.mkdtemp()
    temp1_dir = os.mkdir(os.path.join(temp0_dir, "yotta_targets"))
    temp2_dir = os.mkdir(os.path.join(temp0_dir, "yotta_targets", "target_name"))

    with open(os.path.join(os.path.join(temp0_dir, "yotta_targets", "target_name"), "target.json"), "w") as f:
        f.write(data)

    return temp0_dir

class YottaApiFunctionality(unittest.TestCase):
    def setUp(self):
        self.curr_dir = os.getcwd()

    def tearDown(self):
        pass

    def test_build_with_yotta_invalid_target_name(self):
        res, ret = mbed_yotta_api.build_with_yotta("invalid_name", True, build_to_debug=True)
        self.assertEqual(res, False)

        res, ret = mbed_yotta_api.build_with_yotta("invalid_name", True, build_to_release=True)
        self.assertEqual(res, False)

    def test_get_platform_name_from_yotta_target_invalid_target_file(self):
        temp0_dir = generate_paths_and_write("test")

        os.chdir(temp0_dir)
        result = mbed_yotta_api.get_platform_name_from_yotta_target("target_name")
        self.assertEqual(result, None)
        os.chdir(self.curr_dir)

        shutil.rmtree(temp0_dir)

    def test_get_platform_name_from_yotta_target_missing_keywords(self):
        temp0_dir = generate_paths_and_write("{}")

        os.chdir(temp0_dir)
        result = mbed_yotta_api.get_platform_name_from_yotta_target("target_name")
        self.assertEqual(result, None)
        os.chdir(self.curr_dir)

        shutil.rmtree(temp0_dir)

    def test_get_platform_name_from_yotta_target_missing_targets(self):
        temp0_dir = generate_paths_and_write('{"keywords": []}')

        os.chdir(temp0_dir)
        result = mbed_yotta_api.get_platform_name_from_yotta_target("target_name")
        self.assertEqual(result, None)
        os.chdir(self.curr_dir)

        shutil.rmtree(temp0_dir)

    def test_get_platform_name_from_yotta_target_valid_targets(self):
        temp0_dir = generate_paths_and_write('{"keywords": ["mbed-target:K64F"]}')

        os.chdir(temp0_dir)
        result = mbed_yotta_api.get_platform_name_from_yotta_target("target_name")
        self.assertEqual(result, "K64F")
        os.chdir(self.curr_dir)

        shutil.rmtree(temp0_dir)



"""
mbed SDK
Copyright (c) 2011-2016 ARM Limited

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Author: Przemyslaw Wirkus <Przemyslaw.wirkus@arm.com>
"""
import os
from time import time

from mbed_os_tools.test.mbed_test_api import (
    TEST_RESULT_OK,
    TEST_RESULT_FAIL,
    TEST_RESULT_ERROR,
    TEST_RESULT_SKIPPED,
    TEST_RESULT_UNDEF,
    TEST_RESULT_IOERR_COPY,
    TEST_RESULT_IOERR_DISK,
    TEST_RESULT_IOERR_SERIAL,
    TEST_RESULT_TIMEOUT,
    TEST_RESULT_NO_IMAGE,
    TEST_RESULT_MBED_ASSERT,
    TEST_RESULT_BUILD_FAILED,
    TEST_RESULT_SYNC_FAILED,
    TEST_RESULTS,
    TEST_RESULT_MAPPING,
    RUN_HOST_TEST_POPEN_ERROR,
    get_test_result,
    run_command,
    run_htrun,
    get_testcase_count_and_names,
    get_testcase_utest,
    get_coverage_data,
    get_printable_string,
    get_testcase_summary,
    get_testcase_result,
    get_memory_metrics,
    get_thread_with_max_stack_size,
    get_thread_stack_info_summary,
    log_mbed_devices_in_table,
    get_test_spec,
    get_test_build_properties,
    parse_global_resource_mgr,
    parse_fast_model_connection,
    gt_logger,
)

def run_host_test(image_path,
                  disk,
                  port,
                  build_path,
                  target_id,
                  duration=10,
                  micro=None,
                  reset=None,
                  verbose=False,
                  copy_method=None,
                  program_cycle_s=None,
                  forced_reset_timeout=None,
                  digest_source=None,
                  json_test_cfg=None,
                  max_failed_properties=5,
                  enum_host_tests_path=None,
                  global_resource_mgr=None,
                  fast_model_connection=None,
                  compare_log=None,
                  num_sync_packtes=None,
                  polling_timeout=None,
                  retry_count=1,
                  tags=None,
                  run_app=None):
    """! This function runs host test supervisor (executes mbedhtrun) and checks output from host test process.
    @param image_path Path to binary file for flashing
    @param disk Currently mounted mbed-enabled devices disk (mount point)
    @param port Currently mounted mbed-enabled devices serial port (console)
    @param duration Test case timeout
    @param micro Mbed-enabled device name
    @param reset Reset type
    @param forced_reset_timeout Reset timeout (sec)
    @param verbose Verbose mode flag
    @param copy_method Copy method type (name)
    @param program_cycle_s Wait after flashing delay (sec)
    @param json_test_cfg Additional test configuration file path passed to host tests in JSON format
    @param max_failed_properties After how many unknown properties we will assume test is not ported
    @param enum_host_tests_path Directory where locally defined host tests may reside
    @param num_sync_packtes sync packets to send for host <---> device communication
    @param polling_timeout Timeout in sec for readiness of mount point and serial port of local or remote device
    @param tags Filter list of available devices under test to only run on devices with the provided list
           of tags  [tag-filters tag1,tag]
    @param run_app Run application mode flag (we run application and grab serial port data)
    @param digest_source if None mbedhtrun will be executed. If 'stdin',
           stdin will be used via StdInObserver or file (if
           file name was given as switch option)
    @return Tuple with test results, test output, test duration times, test case results, and memory metrics.
            Return int > 0 if running mbedhtrun process failed.
            Retrun int < 0 if something went wrong during mbedhtrun execution.
    """

    def get_binary_host_tests_dir(binary_path, level=2):
        """! Checks if in binary test group has host_tests directory
        @param binary_path Path to binary in test specification
        @param level How many directories above test host_tests dir exists
        @return Path to host_tests dir in group binary belongs too, None if not found
        """
        try:
            binary_path_norm = os.path.normpath(binary_path)
            current_path_norm = os.path.normpath(os.getcwd())
            host_tests_path = binary_path_norm.split(os.sep)[:-level] + ['host_tests']
            build_dir_candidates = ['BUILD', '.build']
            idx = None

            for build_dir_candidate in build_dir_candidates:
                if build_dir_candidate in host_tests_path:
                    idx = host_tests_path.index(build_dir_candidate)
                    break

            if idx is None:
                msg = 'The following directories were not in the path: %s' % (', '.join(build_dir_candidates))
                raise Exception(msg)

            # Cut /<build dir>/tests/TOOLCHAIN/TARGET
            host_tests_path = host_tests_path[:idx] + host_tests_path[idx+4:]
            host_tests_path = os.sep.join(host_tests_path)
        except Exception as e:
            gt_logger.gt_log_warn("there was a problem while looking for host_tests directory")
            gt_logger.gt_log_tab("level %d, path: %s"% (level, binary_path))
            gt_logger.gt_log_tab(str(e))
            return None

        if os.path.isdir(host_tests_path):
            return host_tests_path
        return None

    if not enum_host_tests_path:
        # If there is -e specified we will try to find a host_tests path ourselves
        #
        # * Path to binary starts from "build" directory, and goes 4 levels
        #   deep: ./build/tests/compiler/toolchain
        # * Binary is inside test group.
        #   For example: <app>/tests/test_group_name/test_dir/*,cpp.
        # * We will search for directory called host_tests on the level of test group (level=2)
        #   or on the level of tests directory (level=3).
        #
        # If host_tests directory is found above test code will will pass it to mbedhtrun using
        # switch -e <path_to_host_tests_dir>
        gt_logger.gt_log("checking for 'host_tests' directory above image directory structure", print_text=verbose)
        test_group_ht_path = get_binary_host_tests_dir(image_path, level=2)
        TESTS_dir_ht_path = get_binary_host_tests_dir(image_path, level=3)
        if test_group_ht_path:
            enum_host_tests_path = test_group_ht_path
        elif TESTS_dir_ht_path:
            enum_host_tests_path = TESTS_dir_ht_path

        if enum_host_tests_path:
            gt_logger.gt_log_tab("found 'host_tests' directory in: '%s'"% enum_host_tests_path, print_text=verbose)
        else:
            gt_logger.gt_log_tab("'host_tests' directory not found: two directory levels above image path checked", print_text=verbose)

    gt_logger.gt_log("selecting test case observer...", print_text=verbose)
    if digest_source:
        gt_logger.gt_log_tab("selected digest source: %s"% digest_source, print_text=verbose)

    # Select who will digest test case serial port data
    if digest_source == 'stdin':
        # When we want to scan stdin for test results
        raise NotImplementedError
    elif digest_source is not None:
        # When we want to open file to scan for test results
        raise NotImplementedError

    # Command executing CLI for host test supervisor (in detect-mode)
    cmd = ["mbedhtrun",
            '-m', micro,
            '-p', port,
            '-f', '"%s"'% image_path,
            ]

    if enum_host_tests_path:
        cmd += ["-e", '"%s"'% enum_host_tests_path]

    if global_resource_mgr:
        # Use global resource manager to execute test
        # Example:
        # $ mbedhtrun -p :9600 -f "tests-mbed_drivers-generic_tests.bin" -m K64F --grm raas_client:10.2.203.31:8000
        cmd += ['--grm', global_resource_mgr]
    else:
        # Use local resources to execute tests
        # Add extra parameters to host_test
        if disk:
            cmd += ["-d", disk]
        if copy_method:
            cmd += ["-c", copy_method]
        if target_id:
            cmd += ["-t", target_id]
        if reset:
            cmd += ["-r", reset]
        if run_app:
            cmd += ["--run"]    # -f stores binary name!

    if fast_model_connection:
        # Use simulator resource manager to execute test
        # Example:
        # $ mbedhtrun -f "tests-mbed_drivers-generic_tests.elf" -m FVP_MPS2_M3 --fm DEFAULT
        cmd += ['--fm', fast_model_connection]
    if compare_log:
        cmd += ['--compare-log', compare_log]
    if program_cycle_s:
        cmd += ["-C", str(program_cycle_s)]
    if forced_reset_timeout:
        cmd += ["-R", str(forced_reset_timeout)]
    if json_test_cfg:
        cmd += ["--test-cfg", '"%s"' % str(json_test_cfg)]
    if num_sync_packtes:
        cmd += ["--sync",str(num_sync_packtes)]
    if tags:
        cmd += ["--tag-filters", tags]
    if polling_timeout:
        cmd += ["-P", str(polling_timeout)]

    gt_logger.gt_log_tab("calling mbedhtrun: %s" % " ".join(cmd), print_text=verbose)
    gt_logger.gt_log("mbed-host-test-runner: started")

    for retry in range(1, 1 + retry_count):
        start_time = time()
        returncode, htrun_output = run_htrun(cmd, verbose)
        end_time = time()
        if returncode < 0:
            return returncode
        elif returncode == 0:
            break
        gt_logger.gt_log("retry mbedhtrun {}/{}".format(retry, retry_count))
    else:
        gt_logger.gt_log("{} failed after {} count".format(cmd, retry_count))

    testcase_duration = end_time - start_time   # Test case duration from reset to {end}
    htrun_output = get_printable_string(htrun_output)
    result = get_test_result(htrun_output)
    result_test_cases = get_testcase_result(htrun_output)
    test_cases_summary = get_testcase_summary(htrun_output)
    max_heap, reserved_heap, thread_stack_info = get_memory_metrics(htrun_output)

    thread_stack_summary = []

    if thread_stack_info:
        thread_stack_summary = get_thread_stack_info_summary(thread_stack_info)

    memory_metrics = {
        "max_heap": max_heap,
        "reserved_heap": reserved_heap,
        "thread_stack_info": thread_stack_info,
        "thread_stack_summary": thread_stack_summary
    }
    get_coverage_data(build_path, htrun_output)

    gt_logger.gt_log("mbed-host-test-runner: stopped and returned '%s'"% result, print_text=verbose)
    return (result, htrun_output, testcase_duration, duration, result_test_cases, test_cases_summary, memory_metrics)



"""
mbed SDK
Copyright (c) 2011-2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import os
import six
import sys
import tempfile
import unittest

from lockfile import LockFile
from mock import patch
from mbed_greentea import mbed_greentea_dlm


class GreenteaDlmFunctionality(unittest.TestCase):
    def setUp(self):
        temp_dir = tempfile.mkdtemp()
        self.home_tools_patch = patch('mbed_os_tools.test.mbed_greentea_dlm.HOME_DIR', temp_dir)
        self.home_local_patch = patch('mbed_greentea.mbed_greentea_dlm.HOME_DIR', temp_dir)
        kettle_dir = os.path.join(temp_dir, mbed_greentea_dlm.GREENTEA_HOME_DIR, mbed_greentea_dlm.GREENTEA_KETTLE)
        self.kettle_tools_patch = patch('mbed_os_tools.test.mbed_greentea_dlm.GREENTEA_KETTLE_PATH', kettle_dir)
        self.kettle_local_patch = patch('mbed_greentea.mbed_greentea_dlm.GREENTEA_KETTLE_PATH', kettle_dir)

        self.home_tools_patch.start()
        self.home_local_patch.start()
        self.kettle_tools_patch.start()
        self.kettle_local_patch.start()

    def tearDown(self):
        self.home_tools_patch.stop()
        self.home_local_patch.stop()
        self.kettle_tools_patch.stop()
        self.kettle_local_patch.stop()

    def test_greentea_home_dir_init(self):
        mbed_greentea_dlm.greentea_home_dir_init()

        path = os.path.join(mbed_greentea_dlm.HOME_DIR, mbed_greentea_dlm.GREENTEA_HOME_DIR)
        self.assertTrue(os.path.exists(path))

    def test_greentea_get_app_sem(self):
        sem, name, uuid = mbed_greentea_dlm.greentea_get_app_sem()
        self.assertIsNotNone(sem)
        self.assertIsNotNone(name)
        self.assertIsNotNone(uuid)

    def test_greentea_get_target_lock(self):
        lock = mbed_greentea_dlm.greentea_get_target_lock("target-id-2")
        path = os.path.join(mbed_greentea_dlm.HOME_DIR, mbed_greentea_dlm.GREENTEA_HOME_DIR, "target-id-2")
        self.assertIsNotNone(lock)
        self.assertEqual(path, lock.path)

    def test_greentea_get_global_lock(self):
        lock = mbed_greentea_dlm.greentea_get_global_lock()
        path = os.path.join(mbed_greentea_dlm.HOME_DIR, mbed_greentea_dlm.GREENTEA_HOME_DIR, "glock.lock")
        self.assertIsNotNone(lock)
        self.assertEqual(path, lock.path)

    def test_get_json_data_from_file_invalid_file(self):
        result = mbed_greentea_dlm.get_json_data_from_file("null_file")
        self.assertIsNone(result)

    def test_get_json_data_from_file_invalid_json(self):
        path = os.path.join(mbed_greentea_dlm.HOME_DIR, "test")

        with open(path, "w") as f:
            f.write("invalid json")

        result = mbed_greentea_dlm.get_json_data_from_file(path)
        self.assertEqual(result, None)

        os.remove(path)

    def test_get_json_data_from_file_valid_file(self):
        path = os.path.join(mbed_greentea_dlm.HOME_DIR, "test")

        with open(path, "w") as f:
            f.write("{}")

        result = mbed_greentea_dlm.get_json_data_from_file(path)
        self.assertEqual(result, {})

        os.remove(path)

    def test_greentea_update_kettle(self):
        uuid = "001"
        mbed_greentea_dlm.greentea_update_kettle(uuid)

        data = mbed_greentea_dlm.get_json_data_from_file(mbed_greentea_dlm.GREENTEA_KETTLE_PATH)
        self.assertIsNotNone(data)
        self.assertIn("start_time", data[uuid])
        self.assertIn("cwd", data[uuid])
        self.assertIn("locks", data[uuid])

        self.assertEqual(data[uuid]["cwd"], os.getcwd())
        self.assertEqual(data[uuid]["locks"], [])

        # Check greentea_kettle_info()
        output = mbed_greentea_dlm.greentea_kettle_info().splitlines()
        line   = output[3]
        self.assertIn(os.getcwd(), line)
        self.assertIn(uuid, line)

        # Test greentea_acquire_target_id
        target_id = "999"
        mbed_greentea_dlm.greentea_acquire_target_id(target_id, uuid)
        data = mbed_greentea_dlm.get_json_data_from_file(mbed_greentea_dlm.GREENTEA_KETTLE_PATH)
        self.assertIn(uuid, data)
        self.assertIn("locks", data[uuid])
        self.assertIn(target_id, data[uuid]["locks"])

        # Test greentea_release_target_id
        mbed_greentea_dlm.greentea_release_target_id(target_id, uuid)
        data = mbed_greentea_dlm.get_json_data_from_file(mbed_greentea_dlm.GREENTEA_KETTLE_PATH)
        self.assertIn(uuid, data)
        self.assertIn("locks", data[uuid])
        self.assertNotIn(target_id, data[uuid]["locks"])

        # Test greentea_acquire_target_id_from_list
        target_id = "999"
        result = mbed_greentea_dlm.greentea_acquire_target_id_from_list([target_id], uuid)
        data = mbed_greentea_dlm.get_json_data_from_file(mbed_greentea_dlm.GREENTEA_KETTLE_PATH)
        self.assertEqual(result, target_id)
        self.assertIn(uuid, data)
        self.assertIn("locks", data[uuid])
        self.assertIn(target_id, data[uuid]["locks"])

        # Check greentea_clean_kettle()
        mbed_greentea_dlm.greentea_clean_kettle(uuid)
        data = mbed_greentea_dlm.get_json_data_from_file(mbed_greentea_dlm.GREENTEA_KETTLE_PATH)
        self.assertEqual(data, {})



"""
mbed SDK
Copyright (c) 2011-2015 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import json
import os
import six
import sys
import tempfile
import unittest

from mock import patch
from mbed_greentea import mbed_report_api

class GreenteaReportApiFunctionality(unittest.TestCase):
    def setUp(self):
        self.test_case_data = {
            "K64F-ARM": {
                "suite": {
                    "testcase_result": {
                        "case-1": {
                            "duration": 1.00,
                            "time_start": 2.00,
                            "time_end": 3.00,
                            "result_text": "FAIL",
                            "passed": 10,
                            "failed": 10
                        },
                        "case-2": {
                            "duration": 1.00,
                            "time_start": 2.00,
                            "time_end": 3.00,
                            "result_text": "SKIPPED",
                            "passed": 10,
                            "failed": 10
                        },
                        "case-3": {
                            "duration": 1.00,
                            "time_start": 2.00,
                            "time_end": 3.00,
                            "result_text": "?",
                            "passed": 10,
                            "failed": 10
                        },
                    },
                    "single_test_output": "OK",
                    "platform_name": "K64F"
                }
            }
        }

        self.test_suite_data = {
            "K64F-ARM": {
                "test-1": {
                    "testcase_result": {
                        "build_path": ".."
                    },
                    "single_test_output": "OK",
                    "single_test_result": "OK",
                    "platform_name": "K64F",
                    "elapsed_time": 1.2,
                    "copy_method": "shell"
                },
                "test-2": {
                    "testcase_result": {
                        "build_path": ".."
                    },
                    "single_test_output": "OK",
                    "single_test_result": "OK",
                    "platform_name": "K64F",
                    "elapsed_time": 1.2,
                    "copy_method": "shell"
                }
            },
            "N32F-ARM": {
                "test-3": {
                    "testcase_result": {
                        "build_path": ".."
                    },
                    "single_test_output": "OK",
                    "single_test_result": "OK",
                    "platform_name": "N32F",
                    "elapsed_time": 1.2,
                    "copy_method": "shell"
                }
            }
        }

    def tearDown(self):
        pass

    def test_export_to_file(self):
        # Invalid filepath
        payload  = "PAYLOAD"
        filepath = "."

        old_stdout = sys.stdout
        sys.stdout = stdout_capture = six.StringIO()
        result = mbed_report_api.export_to_file(filepath, payload)
        sys.stdout = old_stdout

        command_output = stdout_capture.getvalue().splitlines()[0]
        self.assertIn("file failed:", command_output)
        self.assertEqual(result, False)

        # Valid filepath
        temp_file = tempfile.mkstemp("test_file")
        result = mbed_report_api.export_to_file(temp_file[1], payload)
        self.assertTrue(result)

        with open(temp_file[1], 'r') as f:
            read_data = f.read()

        self.assertEqual(read_data, payload)
        os.close(temp_file[0])
        os.remove(temp_file[1])


    def test_exporter_text(self):
        result, result_ = mbed_report_api.exporter_text(self.test_suite_data)

        lines = result.splitlines()
        self.assertIn("target", lines[0])
        self.assertIn("platform_name", lines[0])
        self.assertIn("test suite", lines[0])
        self.assertIn("result", lines[0])
        self.assertIn("K64F", lines[2])
        self.assertIn("test-1", lines[2])
        self.assertIn("test-2", lines[3])
        self.assertIn("test-3", lines[4])
        self.assertIn("OK", lines[2])

    def test_exporter_testcase_test(self):
        result, result_ = mbed_report_api.exporter_testcase_text(self.test_case_data)

        lines = result.splitlines()
        self.assertIn("target", lines[0])
        self.assertIn("platform_name", lines[0])
        self.assertIn("test suite", lines[0])
        self.assertIn("result", lines[0])

        line = lines[2]
        self.assertIn("K64F-ARM", line)
        self.assertIn("suite", line)
        self.assertIn("case-1", line)

    def test_exporter_testcase_junit(self):
        result = mbed_report_api.exporter_testcase_junit(self.test_case_data)
        self.assertIsNotNone(result)

        from xml.etree import ElementTree as ET
        xml = ET.fromstring(result)

        self.assertEqual(xml.tag, "testsuites")
        self.assertEqual(xml.attrib["failures"], "1")
        self.assertEqual(xml.attrib["tests"], "3")
        self.assertEqual(xml.attrib["errors"], "1")
        self.assertEqual(xml.attrib["time"], "3.0")

        child = xml[0]
        self.assertEqual(child.tag, "testsuite")
        self.assertEqual(child.attrib["failures"], "1")
        self.assertEqual(child.attrib["tests"], "3")
        self.assertEqual(child.attrib["errors"], "1")
        self.assertEqual(child.attrib["time"], "3.0")
